@@ -21,13 +21,13 @@
 
 #include "../bomemory/bodummymemory.h"
 #include "bodebug.h"
-#include "rtti.h"
-#include "unit.h"
-#include "unitplugins.h"
-#include "speciestheme.h"
-#include "unitproperties.h"
-#include "upgradeproperties.h"
-#include "cell.h"
+#include "gameengine/rtti.h"
+#include "gameengine/unit.h"
+#include "gameengine/unitplugins.h"
+#include "gameengine/speciestheme.h"
+#include "gameengine/unitproperties.h"
+#include "gameengine/upgradeproperties.h"
+#include "gameengine/cell.h"
 #include "bo3dtools.h"
 
 #include <qstring.h>
@@ -65,9 +65,9 @@ QString BoToolTipCreatorExtended::createToolTip(const BosonItem* item) const
  Unit* u = (Unit*)item;
  tip = i18n("%1\nHealth: %2").arg(u->name()).arg(u->health());
  if (u->isFacility()) {
-	Facility* fac = (Facility*)u;
-	if (!fac->isConstructionComplete()) {
-		tip += i18n("\n%1% constructed").arg((int)fac->constructionProgress());
+	UnitConstruction* c = u->construction();
+	if (!c->isConstructionComplete()) {
+		tip += i18n("\n%1% constructed").arg((int)c->constructionProgress());
 	} else {
 		ProductionPlugin* production = (ProductionPlugin*)u->plugin(UnitPlugin::Production);
 		if (production && production->hasProduction()) {
@@ -170,9 +170,9 @@ QString BoToolTipCreatorDebug::createToolTip(const BosonItem* item) const
  }
 
  if (u->isFacility()) {
-	Facility* fac = (Facility*)u;
-	if (!fac->isConstructionComplete()) {
-		tip += i18n("\n%1% constructed").arg((int)fac->constructionProgress());
+	UnitConstruction* c = u->construction();
+	if (!c->isConstructionComplete()) {
+		tip += i18n("\n%1% constructed").arg((int)c->constructionProgress());
 	} else {
 		ProductionPlugin* production = (ProductionPlugin*)u->plugin(UnitPlugin::Production);
 		if (production && production->hasProduction()) {
