@@ -54,7 +54,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-
 // AB: in contrast to KDE, we allow only (exactly) one MenuBar. No more, no
 // less.
 // AB: we try to use the kpartgui.dtd as far as possible, have a look at it for
@@ -104,7 +103,6 @@ public:
 			boError() << k_funcinfo << "menubar merging failed" << endl;
 			return false;
 		}
-		// TODO: ToolBar tags
 
 		return true;
 	}
@@ -126,19 +124,8 @@ public:
 		}
 		return true;
 	}
-	bool makeGUI(BoUfoMenuBar* bar)
-	{
-		QDomElement root = mDoc.documentElement();
-		if (root.isNull()) {
-			return false;
-		}
-
-		bool ret = makeUfoMenuBar(bar);
-		if (!ret) {
-			boError() << k_funcinfo << "unable to create Ufo MenuBar" << endl;
-		}
-		return ret;
-	}
+	bool makeUfoMenuBar(BoUfoMenuBar* bar, const QString& name);
+	bool makeUfoToolBar(BoUfoToolBar* bar, const QString& name);
 
 protected:
 	bool mergeMenuBars(QDomElement& baseRoot, QDomElement& addRoot)
@@ -158,41 +145,134 @@ protected:
 			return false;
 		}
 
-		// FIXME: baseMenus/addMenus is not correct anymore - these maps
-		// may contains Action items, too
-		QMap<QString, QDomElement> baseMenus;
-		QMap<QString, QDomElement> addMenus;
-		QStringList tags;
-		tags.append("Menu");
-		tags.append("Action");
-		getChildTags(tags, baseMenuBar, addMenuBar, baseMenus, addMenus);
-
-		QMap<QString, QDomElement>::Iterator it;
-		for (it = addMenus.begin(); it != addMenus.end(); ++it) {
-			QString name = it.key();
-			QDomElement addMenu = it.data();
-			if (baseMenus.contains(name)) {
-				boDebug() << "merging menus " << name << endl;
-				QDomElement baseMenu = baseMenus[name];
-				if (baseMenu.tagName() != "Menu") {
-					boWarning() << k_funcinfo << name << " is not a menu. ignoring." << endl;
+		if (!mergeBars(baseMenuBar, addMenuBar)) {
+			boError() << k_funcinfo << "failed merging MenuBars" << endl;
+			return false;
+		}
+		for (QDomNode n = addRoot.firstChild(); !n.isNull(); n = n.nextSibling()) {
+			QDomElement e = n.toElement();
+			if (e.isNull()) {
+				continue;
+			}
+			if (e.tagName() != "ToolBar") {
+				continue;
+			}
+			QDomElement addToolBar = e;
+			QDomElement baseToolBar;
+			QString name = e.attribute("name");
+			for (QDomNode n2 = baseRoot.firstChild(); !n2.isNull() && baseToolBar.isNull(); n2 = n2.nextSibling()) {
+				QDomElement e2 = n2.toElement();
+				if (e2.isNull()) {
 					continue;
 				}
-				if (!mergeMenu(baseMenu, addMenu)) {
+				if (e2.tagName() != "ToolBar") {
+					continue;
+				}
+				if (e2.attribute("name") == name) {
+					baseToolBar = e2;
+				}
+			}
+			if (!baseToolBar.isNull()) {
+				if (!mergeBars(baseToolBar, addToolBar)) {
+					boError() << k_funcinfo << "failed merging ToolBars with name " << name << endl;
 					return false;
 				}
 			} else {
-				boDebug() << "adding " << it.key() << endl;
-				boWarning() << k_funcinfo << "custom menus/toplevel items ignore MergeLocal tags" << endl;
-#warning FIXME
-				// FIXME: append the child before a MergeLocal
-				// tag!!
-				baseMenuBar.appendChild(addMenu);
+				baseMenuBar.parentNode().appendChild(addToolBar);
 			}
 		}
 		return true;
 	}
 
+	bool mergeBars(QDomElement& baseMenuBar, QDomElement& addMenuBar)
+	{
+		QMap<QString, QDomElement> baseTopItems;
+		QMap<QString, QDomElement> addTopItems;
+		QStringList tags;
+		tags.append("Menu");
+		tags.append("Action");
+		getChildTags(tags, baseMenuBar, addMenuBar, baseTopItems, addTopItems);
+
+		QMap<QString, QDomElement>::Iterator it;
+
+		// merge all items that exist in both, addTopItems and
+		// baseTopItems
+		QStringList processedTopItems;
+		for (it = addTopItems.begin(); it != addTopItems.end(); ++it) {
+			QString name = it.key();
+			QDomElement addItem = it.data();
+			if (baseTopItems.contains(name)) {
+				QDomElement baseItem = baseTopItems[name];
+				if (baseItem.tagName() != "Menu") {
+//					boWarning() << k_funcinfo << name << " is not a menu. ignoring." << endl;
+					continue;
+				}
+				if (baseItem.tagName() != addItem.tagName()) {
+					boError() << k_funcinfo << name << " has different tagnames" << endl;
+					continue;
+				}
+				if (!mergeMenu(baseItem, addItem)) {
+					return false;
+				}
+				processedTopItems.append(name);
+			}
+		}
+		// remove all processed items from addTopItems
+		for (QStringList::iterator it = processedTopItems.begin(); it != processedTopItems.end(); ++it) {
+			addTopItems.remove(*it);
+		}
+		processedTopItems.clear();
+
+		// merge all items that exist in addTopItems only
+		// -> place them right before the corresponding MergeLocal tag
+		// but first create a element that contains all addTopItems.
+		QDomElement addItemsElement = addMenuBar.ownerDocument().createElement("Dummy");
+		for (it = addTopItems.begin(); it != addTopItems.end(); ++it) {
+			addItemsElement.appendChild(it.data());
+		}
+#if 0
+		for (QDomNode n = baseMenuBar.firstChild(); !n.isNull(); n = n.nextSibling()) {
+			QDomElement e = n.toElement();
+			if (e.isNull()) {
+				continue;
+			}
+			if (e.tagName() == "MergeLocal") {
+				QString mergeName = e.attribute("name");
+				mergeMenuItems(baseMenuBar, e, addItemsElement, mergeName);
+			}
+		}
+#else
+		mergeMenu(baseMenuBar, addItemsElement);
+#endif
+		return true;
+	}
+
+	void mergeMenuItems(QDomElement& baseMenu, QDomElement& mergeTag, QDomElement& addMenu, const QString& mergeName)
+	{
+		QDomElement it = addMenu.firstChild().toElement();
+		while (!it.isNull()) {
+			QDomElement child = it;
+			// we change it to the next sibling now,
+			// so that we can remove child from this
+			// doc
+			it = it.nextSibling().toElement();
+
+			if (child.tagName() != QString("Action") && child.tagName() != QString("Menu")) {
+				continue;
+			}
+			if (child.attribute("append").isEmpty() != mergeName.isEmpty()) {
+				continue;
+			}
+			if (!mergeName.isEmpty()) {
+				if (child.attribute("append") != mergeName) {
+					continue;
+				}
+			}
+			baseMenu.insertBefore(child.cloneNode(), mergeTag);
+			addMenu.removeChild(child);
+		}
+	}
+
 	void getChildTags(const QStringList& tagNames, QDomElement& baseRoot, QDomElement& addRoot, QMap<QString, QDomElement>& baseMenus, QMap<QString, QDomElement>& addMenus)
 	{
 		QDomNode node = baseRoot.firstChild();
@@ -254,7 +334,6 @@ protected:
 					return false;
 				}
 			} else if (tag == QString("Merge") || tag == QString("MergeLocal") || tag == QString("DefineGroup")) {
-				// AB: Merge is totally unused by us
 				// AB: Merge is totally unused by us
 				element.removeChild(e);
 			}
@@ -262,7 +341,6 @@ protected:
 		return true;
 	}
 
-	bool makeUfoMenuBar(BoUfoMenuBar* bar);
 	bool makeUfoMenu(const QDomElement& menuBar, BoUfoMenuBarMenu* ufoMenu);
 
 private:
@@ -288,28 +366,8 @@ bool BoUfoXMLBuilder::mergeMenu(QDomElement& baseMenu, QDomElement& addMenu)
 			continue;
 		}
 	} else if (tag == QString("MergeLocal")) {
-		QString name = e.attribute("name");
-		QDomElement it = addMenu.firstChild().toElement();
-		while (!it.isNull()) {
-			QDomElement child = it;
-			// we change it to the next sibling now,
-			// so that we can remove child from this
-			// doc
-			it = it.nextSibling().toElement();
-			if (child.tagName() != QString("Action") && child.tagName() != QString("Menu")) {
-				continue;
-			}
-			if (child.attribute("append").isEmpty() != name.isEmpty()) {
-				continue;
-			}
-			if (!name.isEmpty()) {
-				if (child.attribute("append") != name) {
-					continue;
-				}
-			}
-			baseMenu.insertBefore(child.cloneNode(), e);
-			addMenu.removeChild(child);
-		}
+		QString mergeName = e.attribute("name");
+		mergeMenuItems(baseMenu, e, addMenu, mergeName);
 	}
 	e = e.nextSibling().toElement();
  }
@@ -379,8 +437,13 @@ bool BoUfoXMLBuilder::cleanElements(QDomElement& element)
 }
 
 
-bool BoUfoXMLBuilder::makeUfoMenuBar(BoUfoMenuBar* bar)
+bool BoUfoXMLBuilder::makeUfoMenuBar(BoUfoMenuBar* bar, const QString& name)
 {
+ if (!bar) {
+	BO_NULL_ERROR(bar);
+	return false;
+ }
+ Q_UNUSED(name); // atm we support one menubar only
  QDomElement root = mDoc.documentElement();
  QDomElement menuBar = root.namedItem("MenuBar").toElement();
  if (menuBar.isNull()) {
@@ -396,7 +459,37 @@ bool BoUfoXMLBuilder::makeUfoMenuBar(BoUfoMenuBar* bar)
 
  // creation of data structures completed.
  // create actual ufo menubar
- bar->createMenu();
+ bar->createMenuBar();
+
+ return true;
+}
+
+bool BoUfoXMLBuilder::makeUfoToolBar(BoUfoToolBar* bar, const QString& name)
+{
+ if (!bar) {
+	BO_NULL_ERROR(bar);
+	return false;
+ }
+ Q_UNUSED(name); // atm we support one toolbar only
+ QDomElement root = mDoc.documentElement();
+
+ // TODO: honor the "name" attribute of Toolbar tags, atm we support only one
+ // ToolBar
+ QDomElement toolBar = root.namedItem("ToolBar").toElement();
+ if (toolBar.isNull()) {
+	// null toolbar is 100% valid
+	return true;
+ }
+
+ bool ret = makeUfoMenu(toolBar, bar);
+ if (!ret) {
+	boError() << k_funcinfo << "cold not make ToolBar" << endl;
+	return false;
+ }
+
+ // creation of data structures completed.
+ // create actual toolbar
+ bar->createToolBar();
 
  return true;
 }
@@ -449,7 +542,7 @@ bool BoUfoXMLBuilder::makeUfoMenu(const QDomElement& parentElement, BoUfoMenuBar
 }
 
 
-class BoUfoActionDeleter : public ufo::UCollectable
+class BoUfoActionDeleter : public ufo::UBoUfoWidgetDeleter
 {
 public:
 	BoUfoActionDeleter(BoUfoAction* action, ufo::UWidget* widget)
@@ -459,13 +552,69 @@ public:
 	}
 	~BoUfoActionDeleter()
 	{
-		mAction->removeWidget(mWidget, false);
+		doDestructionAction(mAction, mWidget);
 	}
+
+	static void doDestructionAction(BoUfoAction* action, ufo::UWidget* widget)
+	{
+		action->removeWidget(widget, false);
+	}
+
 private:
 	BoUfoAction* mAction;
 	ufo::UWidget* mWidget;
 };
 
+class BoUfoToolBarPushButton : public BoUfoPushButton
+{
+public:
+	BoUfoToolBarPushButton(const QString& text)
+		: BoUfoPushButton(text)
+	{
+		mAction = 0;
+	}
+
+	void setBoUfoAction(BoUfoAction* action)
+	{
+		mAction = action;
+	}
+
+	~BoUfoToolBarPushButton()
+	{
+		BO_CHECK_NULL_RET(mAction);
+		// AB this does NOT delete the widget.
+		BoUfoActionDeleter::doDestructionAction(mAction, ufoWidget());
+	}
+
+private:
+	BoUfoAction* mAction;
+};
+
+class BoUfoToolBarCheckBox : public BoUfoCheckBox
+{
+public:
+	BoUfoToolBarCheckBox(const QString& text)
+		: BoUfoCheckBox(text)
+	{
+		mAction = 0;
+	}
+
+	void setBoUfoAction(BoUfoAction* action)
+	{
+		mAction = action;
+	}
+
+	~BoUfoToolBarCheckBox()
+	{
+		BO_CHECK_NULL_RET(mAction);
+		// AB this does NOT delete the widget.
+		BoUfoActionDeleter::doDestructionAction(mAction, ufoWidget());
+	}
+
+private:
+	BoUfoAction* mAction;
+};
+
 class BoUfoActionPrivate
 {
 public:
@@ -524,12 +673,15 @@ void BoUfoAction::init(BoUfoActionCollection* parent, const QString& text, const
 void BoUfoAction::initShortcut()
 {
  if (qstrcmp(name(), "unnamed") == 0) {
+	boDebug() << k_funcinfo << "won't initialize shortcut for unnamed action" << endl;
 	return;
  }
  if (!mParentCollection) {
+	boDebug() << k_funcinfo << "won't initialize shortcut for action without parent collection" << endl;
 	return;
  }
  if (!mParentCollection->kaccel()) {
+	boDebug() << k_funcinfo << "parent collection has no accel object. delay shortcut initialization until setAccelWidget() was called." << endl;
 	return;
  }
  insertToKAccel(mParentCollection->kaccel());
@@ -597,6 +749,10 @@ void BoUfoAction::slotActivated()
  emit signalActivated();
 }
 
+void BoUfoAction::slotHighlighted()
+{
+}
+
 void BoUfoAction::addWidget(ufo::UWidget* w)
 {
  d->mWidgets.removeRef(w);
@@ -629,12 +785,12 @@ void BoUfoAction::uslotActivated(ufo::UActionEvent*)
 
 void BoUfoAction::uslotHighlighted(ufo::UActionEvent*)
 {
+ slotHighlighted();
 }
 
 
 void BoUfoAction::plug(ufo::UWidget* w)
 {
- // TODO: toolbar?
  ufo::UMenuBar* menuBar = dynamic_cast<ufo::UMenuBar*>(w);
  ufo::UMenu* menu = dynamic_cast<ufo::UMenu*>(w);
  if (menuBar || menu) {
@@ -648,6 +804,17 @@ void BoUfoAction::plug(ufo::UWidget* w)
 
 	w->add(menuItem);
 	addWidget(menuItem);
+ } else {
+	BoUfoToolBarPushButton* button = new BoUfoToolBarPushButton(text());
+	button->ufoWidget()->setFont(w->getFont());
+	button->setBoUfoAction(this);
+	connect(button, SIGNAL(signalClicked()),
+			this, SLOT(slotActivated()));
+	connect(button, SIGNAL(signalHighlighted()),
+			this, SLOT(slotHighlighted()));
+
+	w->add(button->ufoWidget());
+	addWidget(button->ufoWidget());
  }
 }
 
@@ -687,6 +854,18 @@ void BoUfoToggleAction::plug(ufo::UWidget* w)
 
 	w->add(menuItem);
 	addWidget(menuItem);
+ } else {
+	BoUfoToolBarCheckBox* check = new BoUfoToolBarCheckBox(text());
+	check->ufoWidget()->setFont(w->getFont());
+	check->setBoUfoAction(this);
+	check->setChecked(isChecked());
+	connect(check, SIGNAL(signalActivated()),
+			this, SLOT(slotActivated()));
+	connect(check, SIGNAL(signalHighlighted()),
+			this, SLOT(slotHighlighted()));
+
+	w->add(check->ufoWidget());
+	addWidget(check->ufoWidget());
  }
 }
 
@@ -716,8 +895,13 @@ void BoUfoToggleAction::setChecked(bool c)
  QPtrList<ufo::UWidget> list = widgets();
  QPtrListIterator<ufo::UWidget> it(list);
  while (it.current()) {
-	ufo::UCheckBoxMenuItem* checkBox = dynamic_cast<ufo::UCheckBoxMenuItem*>(it.current());
-	if (checkBox) {
+	ufo::UCheckBoxMenuItem* checkBoxItem = dynamic_cast<ufo::UCheckBoxMenuItem*>(it.current());
+	ufo::UCheckBox* checkBox = dynamic_cast<ufo::UCheckBox*>(it.current());
+	if (checkBoxItem) {
+		if (checkBoxItem->isSelected() != isChecked()) {
+			checkBoxItem->setSelected(isChecked());
+		}
+	} else if (checkBox) {
 		if (checkBox->isSelected() != isChecked()) {
 			checkBox->setSelected(isChecked());
 		}
@@ -770,6 +954,10 @@ void BoUfoActionMenu::plug(ufo::UWidget* w)
 
 	w->add(m);
 	addWidget(m);
+ } else {
+	// toolbar
+	// TODO: toolbar
+	boDebug() << k_funcinfo << "submenus not yet supported for toolbars" << endl;
  }
  redoMenus();
 }
@@ -1119,7 +1307,8 @@ void BoUfoActionCollection::clearActions()
  boDebug() << k_funcinfo << endl;
  if (parent() && parent()->inherits("BoUfoManager")) {
 	BoUfoManager* m = (BoUfoManager*)parent();
-	m->setMenuBar(0);
+	m->setMenuBarData(0);
+	m->setToolBarData(0);
  }
  if (d->mParentCollection) {
 	QDictIterator<BoUfoAction> it(d->mActionDict);
@@ -1204,10 +1393,16 @@ bool BoUfoActionCollection::createGUI()
 	return false;
  }
  BoUfoManager* m = (BoUfoManager*)parent();
- m->setMenuBar(0);
+ m->setMenuBarData(0);
+ m->setToolBarData(0);
  BoUfoMenuBar::initMenuBar(m);
- if (!m->menuBar()) {
-	BO_NULL_ERROR(m->menuBar());
+ BoUfoToolBar::initToolBar(m);
+ if (!m->menuBarData()) {
+	BO_NULL_ERROR(m->menuBarData());
+	return false;
+ }
+ if (!m->toolBarData()) {
+	BO_NULL_ERROR(m->toolBarData());
 	return false;
  }
  BoUfoXMLBuilder builder(this);
@@ -1237,7 +1432,19 @@ bool BoUfoActionCollection::createGUI()
 	boError() << k_funcinfo << "failed cleaning the xml doc" << endl;
 	return false;
  }
- return builder.makeGUI(m->menuBar());
+ bool ret = true;
+ if (!builder.makeUfoMenuBar(m->menuBarData(), "MenuBar")) {
+	boError() << k_funcinfo << "building menubar GUI failed" << endl;
+	ret = false;
+ }
+
+ // TODO: honor the "name" attribute of Toolbar tags, atm we support only one
+ // ToolBar
+ if (!builder.makeUfoToolBar(m->toolBarData(), "ToolBar")) {
+	boError() << k_funcinfo << "building toolbar GUI failed" << endl;
+	ret = false;
+ }
+ return ret;
 }
 
 void BoUfoActionCollection::initActionCollection(BoUfoManager* m)
@@ -1259,68 +1466,162 @@ public:
 		mUfoManager = 0;
 		mActionCollection = 0;
 
-		mBar = 0;
+		mMenuBar = 0;
 	}
 
 	BoUfoManager* mUfoManager;
 	BoUfoActionCollection* mActionCollection;
 
-	ufo::UMenuBar* mBar;
+	ufo::UMenuBar* mMenuBar;
+	bool mIsVisible;
 };
 
 BoUfoMenuBar::BoUfoMenuBar(BoUfoManager* parent, const char* name)
 	: BoUfoMenuBarMenu(QString::null, parent, name)
 {
  d = new BoUfoMenuBarPrivate;
+ d->mIsVisible = true;
  d->mUfoManager = parent;
  d->mActionCollection = parent->actionCollection();
 }
 
 BoUfoMenuBar::~BoUfoMenuBar()
 {
- clearUfo();
+ clearUfoMenuBar();
  delete d;
 }
 
+void BoUfoMenuBar::setVisible(bool v)
+{
+ d->mIsVisible = v;
+ if (ufoMenuBar()) {
+	ufoMenuBar()->setVisible(v);
+ }
+}
+
 ufo::UMenuBar* BoUfoMenuBar::ufoMenuBar() const
 {
- return d->mBar;
+ return d->mMenuBar;
 }
 
-void BoUfoMenuBar::clearUfo()
+void BoUfoMenuBar::clearUfoMenuBar()
 {
- if (d->mBar && d->mUfoManager) {
-	d->mUfoManager->setMenuBar(0); // deletes the ufo menubar
-	d->mBar = 0;
+ if (d->mMenuBar && d->mUfoManager) {
+	d->mUfoManager->setMenuBarData(0); // deletes the ufo menubar
+	d->mMenuBar = 0;
  }
 }
 
-void BoUfoMenuBar::createMenu()
+void BoUfoMenuBar::createMenuBar()
 {
  BO_CHECK_NULL_RET(d->mUfoManager);
  BO_CHECK_NULL_RET(d->mUfoManager->rootPane());
  d->mUfoManager->makeContextCurrent();
- clearUfo();
- d->mBar = new ufo::UMenuBar();
- d->mBar->setFont(d->mUfoManager->rootPane()->getFont());
- d->mUfoManager->rootPane()->setMenuBar(d->mBar);
+ clearUfoMenuBar();
+ d->mMenuBar = new ufo::UMenuBar();
+ d->mMenuBar->setVisible(d->mIsVisible);
+ d->mMenuBar->setFont(d->mUfoManager->rootPane()->getFont());
+ d->mUfoManager->rootPane()->setMenuBar(d->mMenuBar);
+ if (d->mMenuBar->getParent()) {
+	// AB: this is a hack.
+	//     if the parent is ever changed in some way in libufo it'll stop
+	//     working.
+	//     actually libufo should make sure that the menubar is always on
+	//     top, but it doesn't, so we ensure this manually here.
+	d->mMenuBar->getParent()->setIndexOf(d->mMenuBar, 0);
+ }
 
  boDebug() << k_funcinfo << "creating submenus" << endl;
- createUfoSubMenu(d->mBar);
+ createUfoMenuBarSubMenu(d->mMenuBar);
 }
 
 void BoUfoMenuBar::initMenuBar(BoUfoManager* m)
 {
  BO_CHECK_NULL_RET(m);
  BO_CHECK_NULL_RET(m->actionCollection());
- if (m->menuBar()) {
+ if (m->menuBarData()) {
 	return;
  }
  BoUfoMenuBar* bar = new BoUfoMenuBar(m, "menubar");
- m->setMenuBar(bar);
+ m->setMenuBarData(bar);
 }
 
 
+class BoUfoToolBarPrivate
+{
+public:
+	BoUfoToolBarPrivate()
+	{
+		mUfoManager = 0;
+		mActionCollection = 0;
+	}
+
+	BoUfoManager* mUfoManager;
+	BoUfoActionCollection* mActionCollection;
+
+	bool mIsVisible;
+};
+
+BoUfoToolBar::BoUfoToolBar(BoUfoManager* parent, const char* name)
+	: BoUfoMenuBarMenu(QString::null, parent, name)
+{
+ d = new BoUfoToolBarPrivate;
+ d->mIsVisible = true;
+ d->mUfoManager = parent;
+ d->mActionCollection = parent->actionCollection();
+}
+
+BoUfoToolBar::~BoUfoToolBar()
+{
+ clearUfoToolBar();
+ delete d;
+}
+
+void BoUfoToolBar::setVisible(bool v)
+{
+ d->mIsVisible = v;
+ if (d->mUfoManager && d->mUfoManager->toolBarContentWidget()) {
+	d->mUfoManager->toolBarContentWidget()->setVisible(d->mIsVisible);
+ }
+}
+
+void BoUfoToolBar::clearUfoToolBar()
+{
+ if (d->mUfoManager && d->mUfoManager->toolBarContentWidget()) {
+	d->mUfoManager->toolBarContentWidget()->removeAllWidgets();
+	d->mUfoManager->toolBarContentWidget()->setVisible(false);
+ }
+}
+
+void BoUfoToolBar::createToolBar()
+{
+ BO_CHECK_NULL_RET(d->mUfoManager);
+ BO_CHECK_NULL_RET(d->mUfoManager->rootPane());
+ BO_CHECK_NULL_RET(d->mUfoManager->toolBarContentWidget());
+ d->mUfoManager->makeContextCurrent();
+ clearUfoToolBar();
+ BO_CHECK_NULL_RET(d->mUfoManager->toolBarContentWidget());
+ d->mUfoManager->toolBarContentWidget()->ufoWidget()->setFont(d->mUfoManager->rootPane()->getFont());
+ d->mUfoManager->toolBarContentWidget()->setVisible(d->mIsVisible);
+
+ boDebug() << k_funcinfo << "creating toolbar submenus" << endl;
+ createUfoToolBarSubMenu(d->mUfoManager->toolBarContentWidget()->ufoWidget());
+}
+
+void BoUfoToolBar::initToolBar(BoUfoManager* m)
+{
+ BO_CHECK_NULL_RET(m);
+ BO_CHECK_NULL_RET(m->actionCollection());
+ if (m->toolBarData()) {
+	return;
+ }
+ BoUfoToolBar* bar = new BoUfoToolBar(m, "toolbar");
+ m->setToolBarData(bar);
+}
+
+
+
+
 
 class BoUfoMenuBarMenuPrivate
 {
@@ -1377,7 +1678,7 @@ const QValueList<BoUfoMenuBarItem*>& BoUfoMenuBarMenu::items() const
  return d->mItems;
 }
 
-void BoUfoMenuBarMenu::createUfoSubMenu(ufo::UWidget* parentWidget)
+void BoUfoMenuBarMenu::createUfoMenuBarSubMenu(ufo::UWidget* parentWidget)
 {
  BO_CHECK_NULL_RET(parentWidget);
 
@@ -1390,7 +1691,43 @@ void BoUfoMenuBarMenu::createUfoSubMenu(ufo::UWidget* parentWidget)
 		menu->setFont(parentWidget->getFont());
 		parentWidget->add(menu);
 
-		m->createUfoSubMenu(menu);
+		m->createUfoMenuBarSubMenu(menu);
+	} else {
+		BoUfoMenuBarItem* item = items()[i];
+		BoUfoAction* action = item->action();
+		if (!action) {
+			// e.g. a separator
+			ufo::UMenuItem* menuItem = 0;
+			menuItem = new ufo::UMenuItem(item->text().latin1());
+			menuItem->setFont(parentWidget->getFont());
+			parentWidget->add(menuItem);
+		} else {
+//			boDebug() << k_funcinfo << "plugging action " << action->name() << endl;
+			action->plug(parentWidget);
+		}
+	}
+ }
+}
+
+void BoUfoMenuBarMenu::createUfoToolBarSubMenu(ufo::UWidget* parentWidget)
+{
+ BO_CHECK_NULL_RET(parentWidget);
+
+ for (unsigned int i = 0; i < itemCount(); i++) {
+	if (items()[i]->isA("BoUfoMenuBarMenu")) {
+		BoUfoMenuBarMenu* m = (BoUfoMenuBarMenu*)items()[i];
+
+		// TODO: we could provide an icon
+#if 0
+		ufo::UMenu* menu = new ufo::UMenu(m->text().latin1());
+		menu->setFont(parentWidget->getFont());
+		parentWidget->add(menu);
+
+		m->createUfoToolBarSubMenu(menu);
+#else
+		boDebug() << k_funcinfo << "submenus not yet supported by toolbar (" << m->text() << ")" << endl;
+		m->createUfoToolBarSubMenu(parentWidget);
+#endif
 	} else {
 		BoUfoMenuBarItem* item = items()[i];
 		BoUfoAction* action = item->action();
