@@ -42,12 +42,14 @@
 #include "boufolayeredpane.h"
 #include "boufofontinfo.h"
 #include "boufoaction.h"
+#include "boufoprofiling.h"
 
 #include <kglobal.h>
 #include <kstandarddirs.h>
 
 #include <qobject.h>
 #include <qdom.h>
+#include <qdir.h>
 
 #include <bodebug.h>
 
@@ -307,11 +309,9 @@ BoUfoManager::BoUfoManager(int w, int h, bool opaque)
 //		tk->putProperty("data_dir", "/usr/local/share/ufo");
 //		tk->putProperty("font_dir", "/usr/local/share/ufo/font");
 	} else {
-		data_dir += "boson/ufo";
-		tk->putProperty("data_dir", data_dir.latin1());
+		data_dir += "boson";
 
-		QString font_dir = data_dir + "/font";
-		tk->putProperty("font_dir", font_dir.latin1());
+		setDataDir(data_dir);
 	}
 
 #if 1
@@ -334,6 +334,12 @@ BoUfoManager::BoUfoManager(int w, int h, bool opaque)
 	// TODO: make sure that it is a UXToolkit
 	boDebug() << k_funcinfo << "have already a toolkit" << endl;
  }
+
+ if (!BoUfoProfiling::profiling()) {
+	BoUfoProfiling::setProfiling(new BoUfoProfiling());
+ }
+ BoUfoProfiling::reference();
+
  mDisplay = new ufo::UXDisplay("dummy");
 
  ufo::URectangle deviceRect(0, 0, w, h);
@@ -346,7 +352,8 @@ BoUfoManager::BoUfoManager(int w, int h, bool opaque)
  mLayeredPaneWidget = 0;
  mContentPane = 0;
  mContentWidget = 0;
- mMenuBar = 0;
+ mMenuBarData = 0;
+ mToolBarData = 0;
  mActionCollection = 0;
 
  makeContextCurrent();
@@ -376,6 +383,17 @@ BoUfoManager::BoUfoManager(int w, int h, bool opaque)
 	rootPane()->setOpaque(opaque);
 	contentWidget()->setOpaque(opaque);
  }
+
+ mToolBarDockWidget = 0;
+ mToolBarContentWidget = 0;
+ if (mRootPane) {
+	mToolBarDockWidget = new ufo::UDockWidget();
+	mToolBarContentWidget = new BoUfoWidget();
+	mToolBarContentWidget->setLayoutClass(BoUfoWidget::UHBoxLayout);
+	mToolBarDockWidget->add(mToolBarContentWidget->ufoWidget());
+	mToolBarContentWidget->setName("boufotoolbarcontent");
+	mRootPane->addDockWidget(mToolBarDockWidget, ufo::TopDockWidgetArea);
+ }
 }
 
 BoUfoManager::~BoUfoManager()
@@ -398,7 +416,7 @@ BoUfoManager::~BoUfoManager()
 	}
  }
  delete mActionCollection;
- setMenuBar(0);
+ setMenuBarData(0);
  if (contentWidget()) {
 	boDebug() << k_funcinfo << "removing all widgets" << endl;
 	contentWidget()->removeAllWidgets();
@@ -410,6 +428,8 @@ BoUfoManager::~BoUfoManager()
  boDebug() << k_funcinfo << "deleting display" << endl;
  delete mDisplay;
 
+ BoUfoProfiling::unreference();
+
  // TODO: do we have to delete the toolkit ?
  // -> there is only a single static instance of it, so that might be tricky.
  // does libufo take care of that?
@@ -417,6 +437,27 @@ BoUfoManager::~BoUfoManager()
  boDebug() << k_funcinfo << "done" << endl;
 }
 
+void BoUfoManager::setDataDir(const QString& data_dir)
+{
+ if (data_dir.isEmpty()) {
+	boWarning() << k_funcinfo << "no data_dir given" << endl;
+	return;
+ }
+ QDir dir(data_dir);
+ if (!dir.exists()) {
+	boWarning() << k_funcinfo << "given data_dir " << data_dir << " does not exist" << endl;
+	return;
+ }
+ QString font_dir = data_dir + "/font";
+ setUfoToolkitProperty("data_dir", data_dir);
+ setUfoToolkitProperty("font_dir", font_dir);
+}
+
+QString BoUfoManager::dataDir() const
+{
+ return ufoToolkitProperty("data_dir");
+}
+
 BoUfoManager* BoUfoManager::currentUfoManager()
 {
  ufo::UToolkit* tk = ufo::UToolkit::getToolkit();
@@ -461,17 +502,31 @@ void BoUfoManager::removeFrame(BoUfoInternalFrame* frame)
  rootPane()->removeFrame(frame->frame());
 }
 
-void BoUfoManager::setMenuBar(BoUfoMenuBar* m)
+void BoUfoManager::setMenuBarData(BoUfoMenuBar* m)
 {
  // warning: BoUfoMenuBar is NOT a BoUfoWidget! therefore it is NOT
  // deleted when the UMenuBar is deleted!
- BoUfoMenuBar* del = mMenuBar;
- mMenuBar = 0; // ~BoUfoMenuBar calls setMenuBar() which deletes mMenuBar
+ BoUfoMenuBar* del = mMenuBarData;
+ mMenuBarData = 0; // ~BoUfoMenuBar calls setMenuBar() which deletes mMenuBarData
  delete del;
  if (mRootPane && mRootPane->getMenuBar()) {
 	mRootPane->setMenuBar(0); // deletes the old menubar
  }
- mMenuBar = m;
+ mMenuBarData = m;
+}
+
+void BoUfoManager::setToolBarData(BoUfoToolBar* m)
+{
+ // warning: BoUfoToolBar is NOT a BoUfoWidget! therefore it is NOT
+ // deleted when the UMenuBar is deleted!
+ BoUfoToolBar* del = mToolBarData;
+ mToolBarData = 0; // ~BoUfoToolBar calls setToolBar() which deletes mToolBarData
+ delete del;
+ if (mToolBarContentWidget) {
+	mToolBarContentWidget->removeAllWidgets();
+	mToolBarContentWidget->setVisible(false);
+ }
+ mToolBarData = m;
 }
 
 void BoUfoManager::makeContextCurrent()
@@ -519,7 +574,7 @@ bool BoUfoManager::sendEvent(QEvent* e)
  //     timer) and then release the button, then the event won't ever get
  //     delivered to the original widget.
  bool updateButtons = false;
- Qt::ButtonState state;
+ Qt::ButtonState state = Qt::NoButton;
  switch (e->type()) {
 	case QEvent::Wheel:
 		state = ((QWheelEvent*)e)->state();
@@ -569,12 +624,14 @@ bool BoUfoManager::sendEvent(QEvent* e)
 		return sendWheelEvent((QWheelEvent*)e);
 	case QEvent::MouseMove:
 		return sendMouseMoveEvent((QMouseEvent*)e);
+	case QEvent::MouseButtonDblClick:
+		// AB: Qt divides press events into simple press events and
+		// dblclick events, libufo does not
+		// -> so we treat both events equally
 	case QEvent::MouseButtonPress:
 		return sendMousePressEvent((QMouseEvent*)e);
 	case QEvent::MouseButtonRelease:
 		return sendMouseReleaseEvent((QMouseEvent*)e);
-	case QEvent::MouseButtonDblClick:
-		return false;
 	case QEvent::KeyPress:
 		return sendKeyPressEvent((QKeyEvent*)e);
 	case QEvent::KeyRelease:
@@ -893,5 +950,12 @@ const BoUfoFontInfo& BoUfoManager::globalFont() const
  return *mGlobalFont;
 }
 
+BoUfoWidget* BoUfoManager::toolBarContentWidget()
+{
+ if (mToolBarContentWidget) {
+	mToolBarContentWidget->setVisible(true);
+ }
+ return mToolBarContentWidget;
+}
 
 
