@@ -1,6 +1,8 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000 Thomas Capricelli (capricel@email.enst.fr)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
+    Copyright (C) 2001-2005 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,23 +23,20 @@
 
 #include <config.h>
 
-#if HAVE_KGAME_VERSION_H
 #include <kgame/kgameversion.h>
-#endif
 
 #include "global.h"
 
-#define BO_TILE_SIZE 48
-#define BO_GL_CELL_SIZE 1.0
-
 // both must be > 0.0:
-#define BO_GL_NEAR_PLANE 1.0f
+#define BO_GL_NEAR_PLANE 0.0625f
 // AB: the largest possible map is 500x500. if you are at one corner (say
 // topleft) you need to see the opposite corner (bottomright), so we need at
 // least sqrt(500*500+500*500)=707. take the camera z into account (say 100) and
 // you need at least sqrt(707*707+100*100)=714.
 // so a value of 1000 should be sufficient for now. (04/02/11)
-#define BO_GL_FAR_PLANE 1000.0f
+// AB: note that using 2^n values leads to higher precision (same for the NEAR
+//     plane: use 2^-m values)
+#define BO_GL_FAR_PLANE 1024.0f
 
 
 #define BOSON_COOKIE 992
@@ -49,8 +48,6 @@
 #define BOSONMAP_TEXMAP_MAGIC_COOKIE QString::fromLatin1("BoTexMap")
 
 
-#define MAXIMAL_ADVANCE_COUNT 19 // maximal value of advanceCount param emitted by Boson::signalAdvance
-
 #define BOSON_MAX_PLAYERS 10 // test if this is working - 2 is tested
 
 #define MAX_GAME_SPEED 25 // the advance period - lower means faster
@@ -60,7 +57,8 @@
 #define MAX_SHIELD_RELOAD_COUNT 10 // number of advance calls after that shield get reloaded by 1
 #define MAX_WEAPONS_PER_UNIT 100
 
-#define BUILD_RANGE 10 * BO_TILE_SIZE // units can be placed within 10 tiles from its factory
+#define BUILD_RANGE 10 // units can be placed within 10 tiles from its factory
+#define REFINERY_FORBID_RANGE 6 // refineries must be at least 6 cells away from resource mines
 
 #define BOSON_MINIMUM_WIDTH 640
 #define BOSON_MINIMUM_HEIGHT 480
@@ -72,9 +70,6 @@
 #define MAX_MAP_HEIGHT 500
 #define MAX_MAP_WIDTH 500
 
-// playfield that gets used by default
-#define DEFAULT_PLAYFIELD QString::fromLatin1("basic2.bpf")
-
 // the placement preview. alpha apply to both previews, the _DISALLOW_COLOR
 // only if the facility can't placed at that point.
 // AB: the red component will alway be 255 anyway.
@@ -84,41 +79,10 @@
 
 // default values
 // used for both BosonConfig and OptionsDialog
-#define DEFAULT_SOUND true
-#define DEFAULT_MUSIC true
-#define DEFAULT_USE_RMB_MOVE true
-#define DEFAULT_USE_MMB_MOVE true
 #define DEFAULT_GAME_SPEED 5
-#define DEFAULT_CURSOR CursorOpenGL
-#define DEFAULT_CURSOR_DIR QString::null // QString::null means use BosonCursor::defaultTheme
-#define DEFAULT_CURSOR_EDGE_SENSITY 20
-#define DEFAULT_ARROW_SCROLL_SPEED 10
-#define DEFAULT_CMD_BUTTONS_PER_ROW 3
-#define DEFAULT_MINIMAP_SCALE 2.0
-#define DEFAULT_MINIMAP_ZOOM 1.0
-#define DEFAULT_UPDATE_INTERVAL 20
-#define DEFAULT_CHAT_SCREEN_REMOVE_TIME 10
-#define DEFAULT_CHAT_SCREEN_MAX_ITEMS 5
-#define DEFAULT_MAGNIFICATION_FILTER GL_LINEAR
-#define DEFAULT_MINIFICATION_FILTER GL_LINEAR
-#define DEFAULT_MIPMAP_MINIFICATION_FILTER GL_LINEAR_MIPMAP_NEAREST
-#define DEFAULT_ALIGN_SELECTION_BOXES true
-#define DEFAULT_RMB_MOVES_WITH_ATTACK true
-#define DEFAULT_MOUSE_WHEEL_ACTION CameraZoom
-#define DEFAULT_MOUSE_WHEEL_SHIFT_ACTION CameraRotate
-#define DEFAULT_DEACTIVATE_WEAPON_SOUNDS false
-#define DEFAULT_USE_LIGHT true // not really supported yet
-#define DEFAULT_USE_MATERIALS false
-#define DEFAULT_USE_MIPMAPS_FOR_MODELS true
-#define DEFAULT_TOOLTIP_UPDATE_PERIOD 300
-#define DEFAULT_TOOLTIP_CREATOR 1 // FIXME: should be BoToolTipCreator::Extended, but I don't want to include the file here
-#define DEFAULT_USE_LOD true
-#define DEFAULT_USE_VBO false  // Nvidia drivers doesn't properly support VBOs :-(
 
 // not in BosonConfig (not necessarily in OptionsDialog):
 #define DEFAULT_CHEAT_MODE true // by default cheating is *enabled* (debugging)
-#define DEFAULT_GROUND_RENDERER "BoDefaultGroundRenderer"
-#define DEFAULT_MESH_RENDERER "BoMeshRendererVertexArray"
 
 #endif
 
