@@ -1,8 +1,8 @@
 /*
     This file is part of the Boson game
     Copyright (C) 1999-2000 Thomas Capricelli (capricel@email.enst.fr)
-    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
-    Copyright (C) 2001-2005 Rivo Laks (rivolaks@hot.ee)
+    Copyright (C) 2001-2006 Andreas Beckermann (b_mann@gmx.de)
+    Copyright (C) 2001-2006 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -45,6 +45,8 @@ class BosonViewData;
 class KGameIO;
 class QDomElement;
 class Boson;
+class BosonLocalPlayerInput;
+class BosonUfoCanvasWidget;
 
 
 
@@ -111,7 +113,7 @@ public:
 	/**
 	 * @return A list of items that are currently in the selection rect
 	 **/
-	BoItemList* items(const PlayerIO* localPlayerIO, const BosonCanvas* canvas) const;
+	BoItemList* items(const BosonUfoCanvasWidget* canvasWidget) const;
 
 signals:
 	void signalVisible(bool);
@@ -139,6 +141,7 @@ public:
 		mControlButton = false;
 		mShiftButton = false;
 		mAltButton = false;
+		mUnitAtEventPos = 0;
 	}
 
 	~BoMouseEvent()
@@ -153,29 +156,37 @@ public:
 	{
 		return mWidgetPos;
 	}
-	void setCanvasVector(const BoVector3Fixed& pos)
+	void setGroundCanvasVector(const BoVector3Fixed& pos)
 	{
-		mCanvasVector = pos;
-		mCanvasPos = QPoint((int)pos.x(), (int)pos.y());
+		mGroundCanvasVector = pos;
+		mGroundCanvasPos = QPoint((int)pos.x(), (int)pos.y());
+	}
+	void setUnitAtEventPos(Unit* unit)
+	{
+		mUnitAtEventPos = unit;
 	}
 
-	const QPoint& canvasPos() const
+	const QPoint& groundCanvasPos() const
 	{
-		return mCanvasPos;
+		return mGroundCanvasPos;
 	}
-	const BoVector3Fixed& canvasVector() const
+	const BoVector3Fixed& groundCanvasVector() const
 	{
-		return mCanvasVector;
+		return mGroundCanvasVector;
+	}
+	Unit* unitAtEventPos() const
+	{
+		return mUnitAtEventPos;
 	}
 
-	void setWorldPos(GLfloat x, GLfloat y, GLfloat z)
+	void setGroundWorldPos(GLfloat x, GLfloat y, GLfloat z)
 	{
 		mX = x;
 		mY = y;
 		mZ = z;
 	}
 
-	void worldPos(GLfloat* x, GLfloat* y, GLfloat* z) const
+	void groundWorldPos(GLfloat* x, GLfloat* y, GLfloat* z) const
 	{
 		*x = mX;
 		*y = mY;
@@ -215,8 +226,8 @@ public:
 
 private:
 	QPoint mWidgetPos;
-	QPoint mCanvasPos;
-	BoVector3Fixed mCanvasVector;
+	QPoint mGroundCanvasPos;
+	BoVector3Fixed mGroundCanvasVector;
 	GLfloat mX;
 	GLfloat mY;
 	GLfloat mZ;
@@ -224,6 +235,8 @@ private:
 	bool mControlButton;
 	bool mShiftButton;
 	bool mAltButton;
+
+	Unit* mUnitAtEventPos;
 };
 
 
@@ -360,7 +373,7 @@ public:
 	 *
 	 * This enters this state.
 	 **/
-	void pressButton(const QPoint& widgetPos, const BoVector3Fixed& canvasVector);
+	void pressButton(const QPoint& widgetPos);
 
 	/**
 	 * Should be called when the mouse button that is responsible for this
@@ -368,7 +381,7 @@ public:
 	 *
 	 * This leaves this state.
 	 **/
-	void releaseButton(const BoMouseEvent& modifiers);
+	void releaseButton(const BoMouseEvent& modifiers, bool doubleRelease = false);
 
 	/**
 	 * Should be called when the mouse is moved
@@ -387,6 +400,14 @@ protected:
 	 **/
 	virtual void action(const BoMouseEvent& modifiers) = 0;
 
+	/**
+	 * Like @ref action, but responds to double clicks (i.e. double
+	 * button releases).
+	 *
+	 * By default this calls @ref action
+	 **/
+	virtual void actionDouble(const BoMouseEvent& modifiers);
+
 	/**
 	 * Called when the button is released and the mouse was moved after the
 	 * button was pressed. Now a corresponding action should be executed.
@@ -420,10 +441,6 @@ protected:
 	 * @return The position where @ref pressButton was called
 	 **/
 	const QPoint& startedAtWidgetPos() const { return mStartWidgetPos; }
-	/**
-	 * @return The position where @ref pressButton was called
-	 **/
-	const BoVector3Fixed& startedAtCanvasVector() const { return mStartCanvasVector; }
 
 	/**
 	 * @return The position where @ref mouseMoved or @ref pressButton
@@ -432,13 +449,6 @@ protected:
 	 * startedAtWidgetPos.
 	 **/
 	const QPoint& currentWidgetPos() const { return mCurrentWidgetPos; }
-	/**
-	 * @return The position where @ref mouseMoved or @ref pressButton
-	 * whichever happened most recently, was called the last time. If no
-	 * call to @ref mouseMoved was made, this equals @ref
-	 * startedAtCanvasVector.
-	 **/
-	const BoVector3Fixed& currentCanvasVector() const { return mCurrentCanvasVector; }
 
 	/**
 	 * @return The amount of pixels in x direction that were moved by the
@@ -454,27 +464,17 @@ protected:
 	 **/
 	int currentWidgetPosDiffY() const { return mCurrentWidgetPosDiffY; }
 
-	/**
-	 * @return The distance the mouse was moved by in the last @ref
-	 * mouseMoved call. Null vector if no such call was made since the last
-	 * @ref pressButton call.
-	 **/
-	const BoVector3Fixed& currentCanvasVectorDiff() const { return mCurrentCanvasVectorDiff; }
-
 private:
 	bool mButtonIsReleased;
 	bool mIsMove;
 	bool mIsCameraAction;
 
 	QPoint mStartWidgetPos;
-	BoVector3Fixed mStartCanvasVector;
 
 	QPoint mCurrentWidgetPos;
-	BoVector3Fixed mCurrentCanvasVector;
 
 	int mCurrentWidgetPosDiffX;
 	int mCurrentWidgetPosDiffY;
-	BoVector3Fixed mCurrentCanvasVectorDiff;
 };
 
 
@@ -544,12 +544,19 @@ public:
 
 	void addChatMessage(const QString& message);
 
+	/**
+	 * Called once after starting a game, to initialize the items that are
+	 * already in the game.
+	 **/
+	bool initializeItems();
 
 	BoLight* light(int id) const;
 	BoLight* newLight();
 	void removeLight(int id);
 
-	void updateOpenGLSettings();
+	void addEffect(unsigned int id, const BoVector3Fixed& pos, bofixed zrot);
+	void addEffectToUnit(int unitid, unsigned int effectid, BoVector3Fixed offset, bofixed zrot);
+	void advanceEffects(int ticks);
 
 	virtual void paint();
 	virtual void paintWidget();
@@ -560,6 +567,7 @@ public slots:
 	 **/
 	void slotReCenterDisplay(const QPoint& pos);
 	void slotCenterHomeBase();
+	void slotChangeCursor(int mode, const QString& dir);
 
 	/**
 	 * In case the unit has been destroyed make sure that it's removed from
@@ -590,12 +598,14 @@ signals:
 	void signalToggleChatVisible();
 	void signalToggleStatusbar(bool);
 	void signalSaveGame();
+	void signalLoadGame();
+	void signalQuicksaveGame();
+	void signalQuickloadGame();
 	void signalEndGame();
 	void signalQuit();
 	void signalEditorChangeLocalPlayer(Player*);
 	void signalEditorHasUndo(const QString&);
 	void signalEditorHasRedo(const QString&);
-	void signalSetUpdateInterval(unsigned int ms);
 
 	/**
 	 * See @ref BosonCursorCollection::signalSetWidgetCursor
@@ -625,14 +635,13 @@ protected:
 	 * @p pos with z=0.0. This is useful for e.g. @ref mapDistance, where
 	 * different z values could deliver wrong values.
 	 **/
-	bool mapCoordinates(const QPoint& pos, GLfloat* posX, GLfloat* posY, GLfloat* posZ, bool useRealDepth = true) const;
-	bool mapCoordinatesToCell(const QPoint& pos, QPoint* cell);
+	bool mapCoordinatesToGround(const QPoint& pos, GLfloat* posX, GLfloat* posY, GLfloat* posZ, bool useRealDepth = true) const;
 	bool mapDistance(int windowDistanceX, int windowDistanceY, GLfloat* dx, GLfloat* dy) const;
 
 	/**
 	 * Set a viewport. Basically the same as glViewport, but you should use
-	 * this instead the standard OpenGL function. The viewport values are
-	 * cached in boson, so that we can easily use it in @ref mapCoordinates.
+	 * this instead the standard OpenGL function, so that we can easily keep
+	 * track of the viewport.
 	 *
 	 * Note that the current matrix is not changed before glViewport is
 	 * calles, so you need to ensure that it is called at the correct
@@ -649,15 +658,6 @@ protected:
 	 **/
 	void moveSelectionRect(const QPoint& widgetPos);
 
-	/**
-	 * Remove a currently drawn selection rect and select all units inside
-	 * this rect.
-	 * @param replace If TRUE the current selection is replaced, otherwise
-	 * the selected units are added to the selection.
-	 * Usually when the player holds the shift key down while selecting.
-	 **/
-	void removeSelectionRect(bool replace);
-
 	/**
 	 * Here the defined action for a wheel event should happen. See
 	 * docs/mouse-big_display.txt for a list of actions that are allowed
@@ -716,6 +716,8 @@ protected:
 
 
 protected slots:
+	void slotPlugLocalPlayerInput();
+	void slotAddMenuInput();
 	void slotResetViewProperties();
 
 	/**
@@ -740,11 +742,11 @@ protected slots:
 	void slotEditorUndo();
 	void slotEditorRedo();
 
+
 	void slotShowLight0Widget();
-	void slotPreferencesApply();
-	void slotUpdateOpenGLSettings();
 
 	void slotScroll(int);
+	void slotCenterOnSelectionGroup(int);
 
 	void slotWidgetResized();
 
@@ -752,6 +754,7 @@ protected slots:
 	void slotWheelEvent(QWheelEvent* e);
 
 	void slotWidgetShown();
+	void slotWidgetHidden();
 
 	void slotGameOver();
 
@@ -833,6 +836,13 @@ protected slots:
 	void slotCommitCameraChanges(int);
 	void slotSetAcceptUserInput(bool);
 
+	/*  Effects  */
+	void slotAddEffect(unsigned int id, const BoVector3Fixed& pos, bofixed zrot);
+	void slotAddEffectToUnit(int unitid, unsigned int effectid, BoVector3Fixed offset, bofixed zrot);
+	void slotAdvanceEffects(int ticks);
+	void slotSetWind(const BoVector3Float& wind);
+	void slotGetWind(BoVector3Float* wind);
+
 protected:
 	void reconnect(const QObject*, const char*, const QObject*, const char*);
 
