@@ -21,6 +21,9 @@
 
 #include "bogroundrendererbase.h"
 
+#include <qmemarray.h>
+#include <qvaluevector.h>
+
 class PlayerIO;
 class QString;
 
@@ -38,21 +41,39 @@ public:
 
 	virtual bool initGroundRenderer();
 
+	virtual void cellFogChanged(int x1, int y1, int x2, int y2);
+	virtual void cellTextureChanged(int x1, int y1, int x2, int y2);
+	virtual void cellHeightChanged(int x1, int y1, int x2, int y2);
+
 protected:
-	virtual void renderVisibleCells(int* cells, unsigned int cellsCount, const BosonMap* map);
+	virtual void renderVisibleCells(int* cells, unsigned int cellsCount, const BosonMap* map, RenderFlags flags);
+	virtual void generateCellList(const BosonMap* map);
+
+	virtual void updateMapCache(const BosonMap* map);
+
+	void clearVBOs();
+	void updateVertexVBO();
+	void updateColorVBO();
 
 private:
-	/**
-	 * Render the @p cells with the current texture.
-	 *
-	 * This is used for texture mixing (blending must be enabled) - first
-	 * the cells are rendered with the first texture, then with the
-	 * second, ...
-	 *
-	 * One could optimize this by using multitexturing for example!
-	 **/
-	unsigned int renderCellsNow(int* cells, int count, int cornersWidth, const float* heightMap, const float* normalMap, const unsigned char* texMapStart);
-	void renderCellColors(int* cells, int count, int width, const float* heightMap);
+	void renderCellColors(int* cells, int count, const BosonMap* map);
+
+	void calculateIndices(int* renderCells, unsigned int cellsCount, const BosonMap* map);
+
+private:
+	const BosonMap* mCurrentMap;
+	unsigned int mVBOVertex;
+	unsigned int mVBONormal;
+	unsigned int mVBOColor;
+
+	unsigned int* mIndicesArray;
+	unsigned int mIndicesArraySize;
+	unsigned int mIndicesCount;
+	bool mIndicesDirty;
+
+	QValueList<int> mIndicesCountList; // used by strips only atm
+
+	QValueVector< QMemArray<unsigned int>* > mTextureIndices;
 };
 
 #endif
