@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2001 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,9 +17,10 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 #include "optionsdialog.h"
+
+#include "../bomemory/bodummymemory.h"
 #include "bosonconfig.h"
 #include "optionswidgets.h"
-
 #include "bosoncursor.h"
 #include "defines.h"
 #include "bodebug.h"
@@ -31,33 +32,34 @@
 
 #include "optionsdialog.moc"
 
-class OptionsDialog::OptionsDialogPrivate
+class OptionsDialogPrivate
 {
 public:
 	OptionsDialogPrivate()
 	{
-		mCursorOptions = 0;
 	}
 	QPtrList<OptionsWidget> mOptionsWidgets;
-	CursorOptions* mCursorOptions;
 };
 
-OptionsDialog::OptionsDialog(bool editor, QWidget* parent, bool modal)
-		: KDialogBase(Tabbed, i18n("Boson Options"), Ok|Apply|Default,
-		Cancel, parent, "bosonoptionsdialog", modal, true)
+OptionsDialog::OptionsDialog(QWidget* parent)
+	: KDialogBase(Tabbed,
+			i18n("Boson Options"),
+			Ok|Apply|Default,
+			Cancel,
+			parent,
+			"bosonoptionsdialog",
+			/*modal*/false,
+			true)
 {
  d = new OptionsDialogPrivate;
 
- if (!editor) {
-	initGeneralPage();
-	initCursorPage();
- }
+ initGeneralPage();
+ initCursorPage();
  initScrollingPage();
- if (!editor) {
-	initSoundsPage();
- }
+ initSoundsPage();
  initOpenGLPage();
- initChatPage(); // in editor for chat messages
+ initWaterPage();
+ initChatPage();
  initToolTipPage();
 }
 
@@ -71,8 +73,6 @@ void OptionsDialog::initGeneralPage()
 {
  QVBox* vbox = addVBoxPage(i18n("&General"));
  GeneralOptions* o = new GeneralOptions(vbox);
- connect(o, SIGNAL(signalCmdBackgroundChanged(const QString&)),
-		this, SIGNAL(signalCmdBackgroundChanged(const QString&)));
  addOptions(o);
 }
 
@@ -83,7 +83,6 @@ void OptionsDialog::initCursorPage()
  connect(o, SIGNAL(signalCursorChanged(int, const QString&)),
 		this, SIGNAL(signalCursorChanged(int, const QString&)));
  addOptions(o);
- d->mCursorOptions = o;
 }
 
 void OptionsDialog::initScrollingPage()
@@ -104,13 +103,18 @@ void OptionsDialog::initOpenGLPage()
 {
  QVBox* vbox = addVBoxPage(i18n("&OpenGL"));
  OpenGLOptions* o = new OpenGLOptions(vbox);
- connect(o, SIGNAL(signalOpenGLSettingsUpdated()),
-		this, SIGNAL(signalOpenGLSettingsUpdated()));
  connect(o, SIGNAL(signalFontChanged(const BoFontInfo&)),
 		this, SIGNAL(signalFontChanged(const BoFontInfo&)));
  addOptions(o);
 }
 
+void OptionsDialog::initWaterPage()
+{
+ QVBox* vbox = addVBoxPage(i18n("&Water"));
+ WaterOptions* o = new WaterOptions(vbox);
+ addOptions(o);
+}
+
 void OptionsDialog::initChatPage()
 {
  QVBox* vbox = addVBoxPage(i18n("C&hat"));
@@ -130,30 +134,6 @@ void OptionsDialog::addOptions(OptionsWidget* o)
  d->mOptionsWidgets.append(o);
 }
 
-void OptionsDialog::setGame(Boson* game)
-{
- QPtrListIterator<OptionsWidget> it(d->mOptionsWidgets);
- for (; it.current(); ++it) {
-	it.current()->setGame(game);
- }
-}
-
-void OptionsDialog::setPlayer(Player* p)
-{
- QPtrListIterator<OptionsWidget> it(d->mOptionsWidgets);
- for (; it.current(); ++it) {
-	it.current()->setLocalPlayer(p);
- }
-}
-
-void OptionsDialog::setCursor(CursorMode mode)
-{
- if (!d->mCursorOptions) {
-	return;
- }
- d->mCursorOptions->setCursor(mode);
-}
-
 void OptionsDialog::slotLoad()
 {
  QPtrListIterator<OptionsWidget> it(d->mOptionsWidgets);
