@@ -50,16 +50,6 @@ public:
 	virtual void load() = 0; // load from current game values
 	virtual void apply() = 0;
 	virtual void setDefaults() = 0; // reset to hardcoded defaults
-
-	Player* localPlayer() const { return mPlayer; }
-	Boson* game() const { return mGame; }
-
-	void setLocalPlayer(Player* p) { mPlayer = p; }
-	void setGame(Boson* g) { mGame = g; }
-
-private:
-	Boson* mGame;
-	Player* mPlayer;
 };
 
 class GeneralOptions : public QVBox, public OptionsWidget
@@ -129,6 +119,7 @@ public:
 	void setCursorEdgeSensity(int);
 	void setRMBScrolling(bool);
 	void setMMBScrolling(bool);
+	void setWheelMoveZoom(bool);
 	void setMouseWheelAction(CameraAction action);
 	void setMouseWheelShiftAction(CameraAction action);
 
@@ -136,6 +127,7 @@ private:
 	KIntNumInput* mArrowSpeed;
 	QCheckBox* mRMBScrolling;
 	QCheckBox* mMMBScrolling;
+	QCheckBox* mWheelMoveZoom;
 	KIntNumInput* mCursorEdgeSensity;
 	QComboBox* mMouseWheelAction;
 	QComboBox* mMouseWheelShiftAction;
@@ -218,10 +210,14 @@ protected:
 	void setCurrentMeshRenderer(const QString&);
 	void setCurrentGroundRenderer(const QString&);
 
+	QString shaderSuffixes();
+	int shaderSuffixesToIndex(const QString& suffixes);
+
+	int shadowMapResolutionToIndex(int resolution);
+	int indexToShadowMapResolution(int index);
+
 protected slots:
 	void slotRenderingSpeedChanged(int);
-	void slotEnableATIDepthWorkaround(bool);
-	void slotATIDepthWorkaroundDefaultValue();
 	void slotShowDetails(bool);
 	void slotChangeFont();
 
@@ -232,9 +228,7 @@ signals:
 private:
 	QComboBox* mRenderingSpeed;
 	QCheckBox* mAlignSelectBoxes;
-	QCheckBox* mEnableATIDepthWorkaround;
 	QComboBox* mResolution;
-	QLineEdit* mATIDepthWorkaroundValue;
 	QVBox* mAdvanced;
 
 	KIntNumInput* mUpdateInterval;
@@ -244,7 +238,9 @@ private:
 	QCheckBox* mUseLight;
 	QCheckBox* mUseMaterials;
 	QComboBox* mGroundRenderer;
-	QCheckBox* mUseGroundShaders;
+	QCheckBox* mUseShaders;
+	QComboBox* mShaderQuality;
+	QComboBox* mShadowQuality;
 	QCheckBox* mUseLOD;
 	QComboBox* mDefaultLOD;
 	QPushButton* mFont;
