@@ -1,7 +1,7 @@
 /*
     This file is part of the Boson game
     Copyright (C) 2002-2005 Rivo Laks (rivolaks@hot.ee)
-    Copyright (C) 2002-2005 Andreas Beckermann (b_mann@gmx.de)
+    Copyright (C) 2002-2006 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,14 +24,14 @@
 #include "../../bomemory/bodummymemory.h"
 #include "../defines.h"
 #include "../bosonconfig.h"
-#include "../player.h"
-#include "../speciestheme.h"
-#include "../bosoncomputerio.h"
-#include "../boson.h"
-#include "../bosoncampaign.h"
-#include "../bosonplayfield.h"
-#include "../bpfdescription.h"
-#include "../bosonmap.h"
+#include "../gameengine/player.h"
+#include "../gameengine/speciestheme.h"
+#include "../gameengine/bosoncomputerio.h"
+#include "../gameengine/boson.h"
+#include "../gameengine/bosoncampaign.h"
+#include "../gameengine/bosonplayfield.h"
+#include "../gameengine/bpfdescription.h"
+#include "../gameengine/bosonmap.h"
 #include "../bosondata.h"
 #include "bosonstartupnetwork.h"
 #include "boufocolorchooser.h"
@@ -49,22 +49,231 @@
 #warning TODO: implement!
 #define HAVE_CHAT_WIDGET 0
 
+class PlayFieldSelection
+{
+public:
+  PlayFieldSelection(BoUfoListBox* campaign, BoUfoListBox* playField)
+  {
+    BO_CHECK_NULL_RET(campaign);
+    BO_CHECK_NULL_RET(playField);
+    mSelectCampaign = campaign;
+    mSelectPlayField = playField;
+    mIsAdmin = true;
+  }
+
+  void addCampaign(const BosonCampaign* campaign)
+  {
+    BO_CHECK_NULL_RET(campaign);
+    QString name = campaign->name();
+    if (name.isEmpty()) {
+      boWarning() << k_funcinfo << "empty campaign name - using <unnamed>" << endl;
+      name = i18n("<unnamed>");
+    }
+    boDebug() << "adding campaign " << name << endl;
+    mIndex2Campaign.insert(mSelectCampaign->count(), campaign);
+    mSelectCampaign->insertItem(name);
+
+    // AB: WARNING: we leave the current item uninitialized!
+    //     we expect the caller to also set the current campaign after adding
+    //     all campaigns!
+  }
+
+  void setAdmin(bool isAdmin)
+  {
+    mIsAdmin = isAdmin;
+    if (mIsAdmin) {
+       mSelectCampaign->setSelectionMode(BoUfoListBox::SingleSelection);
+       mSelectPlayField->setSelectionMode(BoUfoListBox::SingleSelection);
+       mSelectCampaign->setEnabled(true);
+       mSelectPlayField->setEnabled(true);
+    } else {
+       mSelectCampaign->setSelectionMode(BoUfoListBox::NoSelection);
+       mSelectPlayField->setSelectionMode(BoUfoListBox::NoSelection);
+       mSelectCampaign->setEnabled(false);
+       mSelectPlayField->setEnabled(false);
+    }
+  }
+  void updateCurrentPlayField()
+  {
+    int campaign = mSelectCampaign->selectedItem();
+    int playField = mSelectPlayField->selectedItem();
+
+    campaign = QMAX(campaign, 0);
+    campaign = QMIN(campaign, (int)(mIndex2Campaign.count() - 1));
+    setCurrentCampaignWithIndex(campaign);
+
+    playField = QMAX(playField, 0);
+    playField = QMIN(playField, (int)(mIndex2PlayFieldIdentifier.count() - 1));
+    setCurrentPlayFieldWithIndex(playField);
+  }
+
+  void setCurrentCampaignWithIndex(int index)
+  {
+    if (index < 0) {
+      boError() << k_funcinfo << "invalid index " << index << endl;
+      return;
+    }
+    if ((unsigned int)index >= mIndex2Campaign.count()) {
+      boError() << k_funcinfo << "index " << index << " exceeds count " << mIndex2Campaign.count() << endl;
+      return;
+    }
+    if (mIndex2Campaign.count() != mSelectCampaign->count()) {
+      boError() << k_funcinfo << "internal error" << endl;
+      return;
+    }
+    const BosonCampaign* campaign = mIndex2Campaign[index];
+    BO_CHECK_NULL_RET(campaign);
+    int old = mSelectCampaign->selectedItem();
+    if (old != index) {
+      if (!mIsAdmin) {
+          // If client isn't admin, it can't select the campaign, but we still have to
+          //  indicate selected campaign, so we temporarily set selection mode to single
+          mSelectCampaign->setSelectionMode(BoUfoListBox::SingleSelection);
+          mSelectCampaign->blockSignals(true);
+          mSelectCampaign->unselectAll();
+          mSelectCampaign->setItemSelected(index, true);
+          mSelectCampaign->blockSignals(false);
+          mSelectCampaign->setSelectionMode(BoUfoListBox::NoSelection);
+      } else {
+          mSelectCampaign->blockSignals(true);
+          mSelectCampaign->setItemSelected(index, true);
+          mSelectCampaign->blockSignals(false);
+      }
+    }
+    // AB: note that we have to update mSelectPlayField even if the new index
+    // already was selected - setCurrentCampaignWithIndex() is called by the
+    // slot that changes the listbox selection for the campaign!
+
+    mIndex2PlayFieldIdentifier.clear();
+    mSelectPlayField->clear();
+    QStringList list = campaign->playFields();
+    QStringList items;
+    for (unsigned int i = 0; i < list.count(); i++) {
+      BosonPlayField* field = boData->playField(list[i]);
+      if (!field) {
+        BO_NULL_ERROR(field);
+        continue;
+      }
+      int index = items.count();
+      items.append(field->playFieldName());
+      mIndex2PlayFieldIdentifier.insert(index, list[i]);
+    }
+
+    mSelectPlayField->setItems(items);
+
+    setCurrentPlayFieldWithIndex(0);
+  }
+
+  void setCurrentPlayFieldWithIndex(int index)
+  {
+    if (index < 0) {
+      boError() << k_funcinfo << "invalid index " << index << endl;
+      return;
+    }
+    if ((unsigned int)index >= mIndex2PlayFieldIdentifier.count()) {
+      boError() << k_funcinfo << "index " << index << " exceeds count " << mIndex2PlayFieldIdentifier.count() << endl;
+      return;
+    }
+    if (mIndex2PlayFieldIdentifier.count() != mSelectPlayField->count()) {
+      boError() << k_funcinfo << "internal error" << endl;
+      return;
+    }
+
+    int old = mSelectPlayField->selectedItem();
+    if (old != index) {
+      if (!mIsAdmin) {
+          // If client isn't admin, it can't select the map, but we still have to
+          //  indicate selected map, so we temporarily set selection mode to single
+          mSelectPlayField->setSelectionMode(BoUfoListBox::SingleSelection);
+          mSelectPlayField->blockSignals(true);
+          mSelectPlayField->unselectAll();
+          mSelectPlayField->setItemSelected(index, true);
+          mSelectPlayField->blockSignals(false);
+          mSelectPlayField->setSelectionMode(BoUfoListBox::NoSelection);
+      } else {
+          mSelectPlayField->blockSignals(true);
+          mSelectPlayField->setItemSelected(index, true);
+          mSelectPlayField->blockSignals(false);
+      }
+    }
+  }
+
+  void setCurrentPlayFieldWithIdentifier(const QString& identifier)
+  {
+    int campaignIndex = -1;
+    const BosonCampaign* campaign = 0;
+    for (QMap<int, const BosonCampaign*>::iterator it = mIndex2Campaign.begin(); it != mIndex2Campaign.end() && !campaign; ++it) {
+      const BosonCampaign* c = it.data();
+      if (!c) {
+        BO_NULL_ERROR(c);
+        continue;
+      }
+      if (c->playFields().contains(identifier)) {
+        campaignIndex = it.key();
+        campaign = c;
+      }
+    }
+    if (!campaign || campaignIndex < 0) {
+      boError() << k_funcinfo << "cannot find campaign for playfield " << identifier << endl;
+      return;
+    }
+
+    setCurrentCampaignWithIndex(campaignIndex);
+
+    int playFieldIndex = -1;
+    for (QMap<int, QString>::iterator it = mIndex2PlayFieldIdentifier.begin(); it != mIndex2PlayFieldIdentifier.end() && playFieldIndex < 0; ++it) {
+      if (it.data() == identifier) {
+        playFieldIndex = it.key();
+      }
+    }
+    if (playFieldIndex < 0) {
+      boError() << k_funcinfo << "could not find playfield " << identifier << endl;
+      return;
+    }
+    setCurrentPlayFieldWithIndex(playFieldIndex);
+  }
+
+  QString playFieldIdentifier() const
+  {
+    if (mSelectPlayField->selectedItem() < 0) {
+      return QString::null;
+    }
+    if (!mIndex2PlayFieldIdentifier.contains(mSelectPlayField->selectedItem())) {
+      return QString::null;
+    }
+    QString identifier = mIndex2PlayFieldIdentifier[mSelectPlayField->selectedItem()];
+    return identifier;
+  }
+
+private:
+  bool mIsAdmin;
+  BoUfoListBox* mSelectCampaign;
+  BoUfoListBox* mSelectPlayField;
+
+  QMap<int, const BosonCampaign*> mIndex2Campaign;
+  QMap<int, QString> mIndex2PlayFieldIdentifier;
+};
+
 class BoUfoNewGameWidgetPrivate
 {
 public:
     BoUfoNewGameWidgetPrivate()
     {
-        mLocalPlayer = 0;
+      mPlayFieldSelection = 0;
+      mLocalPlayer = 0;
 
-        mPlayerColor = 0;
+      mPlayerColor = 0;
     }
 
+    PlayFieldSelection* mPlayFieldSelection;
     QMap<int, KPlayer*> mItem2Player;
     QMap<int, QString> mItem2Map; // "item" is an index in the listbox now
 
     QMap<int, QString> mSpeciesIndex2Identifier;
     QMap<int, QString> mSpeciesIndex2Comment;
 
+    QMap<int, int> mSideIndex2Side; // index in combobox -> Player::bosonId() of the side
+
     QGuardedPtr<Player> mLocalPlayer;
 
     BoUfoColorChooser* mPlayerColor;
@@ -88,6 +297,8 @@ BoUfoNewGameWidget::BoUfoNewGameWidget(BosonStartupNetwork* interface)
  mInited = false; // Will become true once localplayer gets added
  d->mComputerPlayerNumber = 1;
 
+ d->mPlayFieldSelection = new PlayFieldSelection(mSelectCampaign, mSelectMap);
+
  d->mPlayerColor = new BoUfoColorChooser();
  mPlayerColorContainer->setLayoutClass(BoUfoWidget::UHBoxLayout);
  mPlayerColorContainer->addWidget(d->mPlayerColor);
@@ -95,6 +306,8 @@ BoUfoNewGameWidget::BoUfoNewGameWidget(BosonStartupNetwork* interface)
         this, SLOT(slotPlayerColorChanged(int)));
 
  initPlayFields();
+ d->mPlayFieldSelection->setCurrentCampaignWithIndex(0);
+ d->mPlayFieldSelection->setCurrentPlayFieldWithIndex(0);
  initSpecies();
 
  connect(networkInterface(), SIGNAL(signalStartGameClicked()),
@@ -105,6 +318,8 @@ BoUfoNewGameWidget::BoUfoNewGameWidget(BosonStartupNetwork* interface)
         this, SLOT(slotNetPlayerLeftGame(KPlayer*)));
  connect(networkInterface(), SIGNAL(signalSpeciesChanged(Player*)),
         this, SLOT(slotNetSpeciesChanged(Player*)));
+ connect(networkInterface(), SIGNAL(signalSideChanged(Player*)),
+        this, SLOT(slotNetSideChanged(Player*)));
  connect(networkInterface(), SIGNAL(signalTeamColorChanged(Player*)),
         this, SLOT(slotNetColorChanged(Player*)));
  connect(networkInterface(), SIGNAL(signalPlayFieldChanged(BosonPlayField*)),
@@ -143,12 +358,7 @@ BoUfoNewGameWidget::~BoUfoNewGameWidget()
     boError() << k_funcinfo << "FIXME: Boson already deleted!" << endl;
  }
  boConfig->saveLocalPlayerColor(mLocalPlayerColor);
- QString playFieldIdentifier;
- if (mSelectMap->selectedItem() >= 0) {
-    if (d->mItem2Map.contains(mSelectMap->selectedItem())) {
-        playFieldIdentifier = d->mItem2Map[mSelectMap->selectedItem()];
-    }
- }
+ QString playFieldIdentifier = d->mPlayFieldSelection->playFieldIdentifier();
  if (!playFieldIdentifier.isEmpty()) {
     boConfig->saveLocalPlayerMap(playFieldIdentifier);
  }
@@ -204,7 +414,7 @@ void BoUfoNewGameWidget::initPlayFields()
  // Add default campaign
  BosonCampaign* defaultCampaign = boData->campaign(QString::fromLatin1(""));
  BO_CHECK_NULL_RET(defaultCampaign);
- initPlayFields(defaultCampaign);
+ d->mPlayFieldSelection->addCampaign(defaultCampaign);
 
  // Add other campaigns
  for (unsigned int i = 0; i < list.count(); i++) {
@@ -213,36 +423,10 @@ void BoUfoNewGameWidget::initPlayFields()
         BO_NULL_ERROR(campaign);
         continue;
     }
-    initPlayFields(campaign);
+    d->mPlayFieldSelection->addCampaign(campaign);
  }
 }
 
-void BoUfoNewGameWidget::initPlayFields(BosonCampaign* campaign)
-{
- boDebug() << k_funcinfo << endl;
- BO_CHECK_NULL_RET(campaign);
- QString campaingPrefix;
- if (campaign->identifier().isEmpty()) {
-    boDebug() << k_funcinfo << "default campaign" << endl;
- } else {
-    boDebug() << k_funcinfo << campaign->identifier() << endl;
-    campaingPrefix = campaign->name() + ": " ;
- }
- QStringList list = campaign->playFields();
- QStringList items = mSelectMap->items();
- for (unsigned int i = 0; i < list.count(); i++) {
-    BosonPlayField* field = boData->playField(list[i]);
-    if (!field) {
-        BO_NULL_ERROR(field);
-        continue;
-    }
-    int index = items.count();
-    items.append(campaingPrefix + field->playFieldName());
-    d->mItem2Map.insert(index, list[i]);
- }
- mSelectMap->setItems(items);
-}
-
 void BoUfoNewGameWidget::initSpecies()
 {
  boDebug() << k_funcinfo << endl;
@@ -327,54 +511,50 @@ void BoUfoNewGameWidget::slotNetStart()
     return;
  }
  // Check for map
- if (mSelectMap->selectedItem() < 0) {
+ if (d->mPlayFieldSelection->playFieldIdentifier().isEmpty()) {
     KMessageBox::sorry(0, i18n("No map selected. Select a map first!"));
     return;
  }
  // Check for valid players count
- if (boGame->playerCount() > mMaxPlayers) {
+ if (boGame->gamePlayerCount() > mMaxPlayers) {
     KMessageBox::sorry(0, i18n("There are too many players in game.\n"
             "Current map supports only %1 players, currently, there are %2 players in the game.\n"
-            "Please remove some players.").arg(mMaxPlayers).arg(boGame->playerCount()),
+            "Please remove some players.").arg(mMaxPlayers).arg(boGame->gamePlayerCount()),
             i18n("Too many players"));
- } else if (boGame->playerCount() < mMinPlayers) {
+ } else if (boGame->gamePlayerCount() < mMinPlayers) {
     KMessageBox::sorry(0, i18n("There are too few players in game.\n"
             "Current map requires at least %1 players, currently, there are only %2 players in the game.\n"
-            "Please add some players.").arg(mMinPlayers).arg(boGame->playerCount()),
+            "Please add some players.").arg(mMinPlayers).arg(boGame->gamePlayerCount()),
             i18n("Too few players"));
  } else {
     // All good
     //slotPlayerNameChanged();
     // Check if each player has unique team color
-    for (unsigned int i = 0; i < boGame->playerCount() - 1; i++) {
-        Player* p = (Player*)boGame->playerList()->at(i);
+    for (unsigned int i = 0; i < boGame->gamePlayerCount() - 1; i++) {
+        Player* p = boGame->gamePlayerList()->at(i);
         if (!p) {
             BO_NULL_ERROR(p);
             continue;
         }
-        for (unsigned int j = i + 1; j < boGame->playerCount(); j++) {
-            Player* p2 = (Player*)boGame->playerList()->at(j);
+        for (unsigned int j = i + 1; j < boGame->gamePlayerCount(); j++) {
+            Player* p2 = boGame->gamePlayerList()->at(j);
             if (!p2) {
                 BO_NULL_ERROR(p2);
                 continue;
             }
             if (p->teamColor() == p2->teamColor()) {
-                KMessageBox::sorry(0, i18n("Cannot start game - player %1 (id=%2) and %3 (id=%4) have the same teamcolor.\nThis is not allowed!").arg(p->name()).arg(p->id()).arg(p2->name()).arg(p2->id()));
+                KMessageBox::sorry(0, i18n("Cannot start game - player %1 (id=%2) and %3 (id=%4) have the same teamcolor.\nThis is not allowed!").arg(p->name()).arg(p->bosonId()).arg(p2->name()).arg(p2->bosonId()));
                 return;
             }
         }
     }
-    if (mSelectMap->selectedItem() < 0) {
+
+    BosonPlayField* field = 0;
+    QString identifier = d->mPlayFieldSelection->playFieldIdentifier();
+    if (identifier.isNull()) {
         KMessageBox::sorry(0, i18n("You have to select a map first"));
         return;
     }
-
-    BosonPlayField* field = 0;
-    if (!d->mItem2Map.contains(mSelectMap->selectedItem())) {
-        KMessageBox::sorry(0, i18n("The selected item seems not to be a valid map. Please select a different map.\n(if you believe you have selected a valid map, you might have encountered a bug)"));
-        return;
-    }
-    QString identifier = d->mItem2Map[mSelectMap->selectedItem()];
     field = boData->playField(identifier);
 
     if (!field) {
@@ -382,8 +562,16 @@ void BoUfoNewGameWidget::slotNetStart()
         return;
     }
 
-    // Send start message over net
-    if (!networkInterface()->sendNewGame(field, false)) {
+    bool ret = true;
+    if (ret) {
+      if (boGame->isAdmin()) {
+        ret = networkInterface()->addNeutralPlayer(false);
+      }
+    }
+    if (ret) {
+      ret = networkInterface()->sendNewGame(field, false);
+    }
+    if (!ret) {
         KMessageBox::sorry(0, i18n("The selected map could not be saved to a network stream. This might be due to a broken map file, or due to a boson bug.\nAnyway - the game cannot be started."));
         return;
     } else {
@@ -419,16 +607,16 @@ void BoUfoNewGameWidget::slotNetPlayerJoinedGame(KPlayer* p)
         // Localplayer added AI player
         boGame->slotAddChatSystemMessage("Boson", i18n("You added AI player %1 to the game").arg(p->name()));
     } else {
-        int gameID = KGameMessage::rawGameId(p->id());  // gameID == clientID
-        KPlayer* mainPlayer = boGame->findPlayer(KGameMessage::createPlayerId(1, gameID));
+        int gameID = KGameMessage::rawGameId(p->kgameId());  // gameID == clientID
+        KPlayer* mainPlayer = boGame->findPlayerByKGameId(KGameMessage::createPlayerId(1, gameID));
         if (mainPlayer == p) {
             // Another client connected
             boGame->slotAddChatSystemMessage("Boson", i18n("%1 joined the game").arg(p->name()));
             if (boGame->isAdmin()) {
                 boDebug() << k_funcinfo << "new client connected - sending current playfield" << endl;
-                int item = mSelectMap->selectedItem();
-                if (item >= 0) {
-                    networkInterface()->sendChangePlayField(d->mItem2Map[item]);
+                QString identifier = d->mPlayFieldSelection->playFieldIdentifier();
+                if (!identifier.isEmpty()) {
+                    networkInterface()->sendChangePlayField(identifier);
                 }
             }
         } else if(!mainPlayer) {
@@ -440,8 +628,8 @@ void BoUfoNewGameWidget::slotNetPlayerJoinedGame(KPlayer* p)
             boGame->slotAddChatSystemMessage("Boson", i18n("%1 added AI player %2 to the game").arg(mainPlayer->name()).arg(p->name()));
         }
     }
-    for (int i = 0; i < (int)boGame->playerCount() - 1; i++) {
-        Player* p2 = (Player*)boGame->playerList()->at(i);
+    for (int i = 0; i < (int)boGame->gamePlayerCount() - 1; i++) {
+        Player* p2 = boGame->gamePlayerList()->at(i);
         if (p2 == (Player*)p) {
             continue;
         }
@@ -456,11 +644,12 @@ void BoUfoNewGameWidget::slotNetPlayerJoinedGame(KPlayer* p)
 
  updateColors();
  playerCountChanged();
+ possibleSidesChanged();
 }
 
 void BoUfoNewGameWidget::slotNetPlayerLeftGame(KPlayer* p)
 {
- boDebug() << k_funcinfo << "there are " << boGame->playerList()->count() << " players in game now" << endl;
+ boDebug() << k_funcinfo << "there are " << boGame->allPlayerList()->count() << " players in game now with " << boGame->gamePlayerList()->count() << " game players" << endl;
 
  if (((Player*)p)->isNeutralPlayer()) {
     // Neutral player shouldn't be removed because it's not added until the game
@@ -507,6 +696,12 @@ void BoUfoNewGameWidget::slotNetSpeciesChanged(Player* p)
  }
 }
 
+void BoUfoNewGameWidget::slotNetSideChanged(Player* p)
+{
+ boDebug() << k_funcinfo << endl;
+ possibleSidesChanged();
+}
+
 void BoUfoNewGameWidget::slotNetColorChanged(Player* p)
 {
  // Update color selector widget
@@ -517,38 +712,17 @@ void BoUfoNewGameWidget::slotNetPlayFieldChanged(BosonPlayField* field)
 {
  BO_CHECK_NULL_RET(field);
  boDebug() << k_funcinfo << "id: " << field->identifier() << endl;
+ d->mPlayFieldSelection->setCurrentPlayFieldWithIdentifier(field->identifier());
+ if (d->mPlayFieldSelection->playFieldIdentifier() != field->identifier()) {
+   boError() << k_funcinfo << "could not set correct playfield " << field->identifier() << endl;
+   field = boData->playField(d->mPlayFieldSelection->playFieldIdentifier());
+   BO_CHECK_NULL_RET(field);
+ }
  if (!field->isPreLoaded()) {
     // well, if that happens - something evil must have happened!
     boWarning() << k_funcinfo << "playfield not yet preloaded?!" << endl;
     field->preLoadPlayField(field->identifier());
  }
- QStringList list = boData->availablePlayFields();
- QMap<int, QString>::Iterator it;
- int item = -1;
- for (it = d->mItem2Map.begin(); it != d->mItem2Map.end() && (item < 0); ++it) {
-    if (it.data() == field->identifier()) {
-        item = it.key();
-        break;
-    }
- }
- if (item < 0) {
-    boError() << k_funcinfo << "Cannot find playfield item for " << field->identifier() << endl;
- } else {
-    if (!boGame->isAdmin()) {
-        // If client isn't admin, it can't select the map, but we still have to
-        //  indicate selected map, so we temporarily set selection mode to single
-        mSelectMap->setSelectionMode(BoUfoListBox::SingleSelection);
-        mSelectMap->blockSignals(true);
-        mSelectMap->unselectAll();
-        mSelectMap->setItemSelected(item, true);
-        mSelectMap->blockSignals(false);
-        mSelectMap->setSelectionMode(BoUfoListBox::NoSelection);
-    } else {
-        mSelectMap->blockSignals(true);
-        mSelectMap->setItemSelected(item, true);
-        mSelectMap->blockSignals(false);
-    }
- }
  mMinPlayers = field->information()->minPlayers();
  mMaxPlayers = field->information()->maxPlayers();
  boDebug() << "minPlayers: " << mMinPlayers << " ; maxPlayers: " << mMaxPlayers << endl;
@@ -566,6 +740,7 @@ void BoUfoNewGameWidget::slotNetPlayFieldChanged(BosonPlayField* field)
  if (!description) {
     boWarning() << k_funcinfo << "NULL description" << endl;
     BosonPlayField::preLoadAllPlayFields();
+    description = field->description();
     if (!description) {
         boError() << k_funcinfo << "unable to load the description" << endl;
         return;
@@ -580,6 +755,7 @@ void BoUfoNewGameWidget::slotNetPlayFieldChanged(BosonPlayField* field)
     mMapDescription->setText(description->comment());
  }
  playerCountChanged();
+ possibleSidesChanged();
 }
 
 void BoUfoNewGameWidget::slotNetPlayerNameChanged(Player* p)
@@ -614,7 +790,7 @@ void BoUfoNewGameWidget::slotNetPlayerNameChanged(Player* p)
         return;
     }
  }
- boWarning() << k_funcinfo << "could not find player " << p->id() << endl;
+ boWarning() << k_funcinfo << "could not find player " << p->bosonId() << endl;
 }
 
 void BoUfoNewGameWidget::slotNetSetLocalPlayer(Player* p)
@@ -653,7 +829,7 @@ void BoUfoNewGameWidget::slotPlayerNameChanged()
  // Non-admin players can only change names of non-virtual players
  if (!boGame->isAdmin() && mSelectedPlayer->isVirtual()) {
     // This shouldn't happen, name lineedit should be read-only in this case
-    boError() << k_funcinfo << "Can't set name for player " << p->id() << endl;
+    boError() << k_funcinfo << "Can't set name for player " << p->bosonId() << endl;
     mPlayerName->setText(p->name());
     return;
  }
@@ -677,7 +853,7 @@ void BoUfoNewGameWidget::slotPlayerColorChanged(int index)
  }
  BO_CHECK_NULL_RET(p);
  if (!boGame->isAdmin() && mSelectedPlayer->isVirtual()) {
-    boError() << k_funcinfo << "Can't set color for player " << mSelectedPlayer->id() << endl;
+    boError() << k_funcinfo << "Can't set color for player " << mSelectedPlayer->bosonId() << endl;
     KMessageBox::sorry(0, i18n("Only ADMIN can change color of other players!"));
     return;
  }
@@ -697,22 +873,20 @@ void BoUfoNewGameWidget::slotPlayFieldSelected(int first, int last)
  // one of these is selected _currently_. better just ignore them.
  Q_UNUSED(first);
  Q_UNUSED(last);
- int selected = mSelectMap->selectedItem();
- slotPlayFieldSelected(selected);
+ d->mPlayFieldSelection->updateCurrentPlayField();
+ QString identifier = d->mPlayFieldSelection->playFieldIdentifier();
+ networkInterface()->sendChangePlayField(identifier);
 }
 
-void BoUfoNewGameWidget::slotPlayFieldSelected(int index)
+void BoUfoNewGameWidget::slotCampaignSelected(int first, int last)
 {
- boDebug() << k_funcinfo << endl;
- if (index < 0) {
-   boError() << k_funcinfo << "index < 0: " << index << endl;
-   return;
- }
- if (!d->mItem2Map.contains(index)) {
-    boWarning() << k_funcinfo << "invalid item index " << index << endl;
-    return;
- }
- networkInterface()->sendChangePlayField(d->mItem2Map[index]);
+ // AB: first and last are barely usable. there is no way to find out _which_
+ // one of these is selected _currently_. better just ignore them.
+ Q_UNUSED(first);
+ Q_UNUSED(last);
+ d->mPlayFieldSelection->updateCurrentPlayField();
+ QString identifier = d->mPlayFieldSelection->playFieldIdentifier();
+ networkInterface()->sendChangePlayField(identifier);
 }
 
 void BoUfoNewGameWidget::slotPlayerSpeciesChanged(int index)
@@ -725,7 +899,7 @@ void BoUfoNewGameWidget::slotPlayerSpeciesChanged(int index)
     p = localPlayer();
  }
  if (!boGame->isAdmin() && mSelectedPlayer->isVirtual()) {
-    boError() << k_funcinfo << "Can't set species for player " << p->id() << endl;
+    boError() << k_funcinfo << "Can't set species for player " << p->bosonId() << endl;
     return;
  }
 
@@ -736,6 +910,27 @@ void BoUfoNewGameWidget::slotPlayerSpeciesChanged(int index)
  networkInterface()->sendChangeSpecies(p, d->mSpeciesIndex2Identifier[index], p->speciesTheme()->teamColor());
 }
 
+void BoUfoNewGameWidget::slotPlayerSideChanged(int index)
+{
+ boDebug() << k_funcinfo << endl;
+ BO_CHECK_NULL_RET(localPlayer());
+
+ Player* p = mSelectedPlayer;
+ if (!p) {
+    p = localPlayer();
+ }
+ if (!boGame->isAdmin() && mSelectedPlayer->isVirtual()) {
+    boError() << k_funcinfo << "Can't set side for player " << p->bosonId() << endl;
+    return;
+ }
+
+ if (index >= (int)d->mSideIndex2Side.count()) {
+    boError() << k_funcinfo << "invalid index " << index << endl;
+    return;
+ }
+ networkInterface()->sendChangeSide(p, d->mSideIndex2Side[index]);
+}
+
 void BoUfoNewGameWidget::slotAddComputerPlayer()
 {
  BO_CHECK_NULL_RET(boGame);
@@ -813,10 +1008,13 @@ void BoUfoNewGameWidget::slotPlayerSelected(int index)
  QMap<int, QString>::Iterator it;
  for (it = d->mSpeciesIndex2Identifier.begin(); it != d->mSpeciesIndex2Identifier.end(); it++) {
     if (it.data() == mSelectedPlayer->speciesTheme()->identifier()) {
+        mPlayerSpecies->blockSignals(true);
         mPlayerSpecies->setCurrentItem(it.key());
+        mPlayerSpecies->blockSignals(false);
         break;
     }
  }
+ possibleSidesChanged();
 
  // Enable/disable some stuff
  bool canChange = false;
@@ -826,6 +1024,7 @@ void BoUfoNewGameWidget::slotPlayerSelected(int index)
  mPlayerName->setEditable(canChange);
  d->mPlayerColor->setEnabled(canChange);
  mPlayerSpecies->setEnabled(canChange);
+ mPlayerDesiredSide->setEnabled(canChange);
  mRemovePlayer->setEnabled(canChange && (mSelectedPlayer != localPlayer()));
 }
 
@@ -875,16 +1074,88 @@ void BoUfoNewGameWidget::playerCountChanged()
  // Maybe also disable/enable start button if player count is invalid?
 }
 
+void BoUfoNewGameWidget::possibleSidesChanged()
+{
+ boDebug() << k_funcinfo << endl;
+ BO_CHECK_NULL_RET(boGame);
+ mPlayerDesiredSide->clear();
+ d->mSideIndex2Side.clear();
+
+ if (!mSelectedPlayer) {
+   boDebug() << k_funcinfo << "no player selected" << endl;
+   return;
+ }
+
+ QStringList sides;
+
+#warning TODO: read available sides from map
+ // TODO: insert possible sides of the map
+ unsigned int maxPlayers = BOSON_MAX_PLAYERS;
+ maxPlayers = QMIN(maxPlayers, mMaxPlayers);
+ maxPlayers = QMAX(maxPlayers, mMinPlayers);
+ for (unsigned int i = 0; i < maxPlayers; i++) {
+   int id = 128 + i;
+   Player* p = (Player*)boGame->findPlayerByUserId(id);
+   if (p && p != mSelectedPlayer) {
+     continue;
+   }
+   sides.append(i18n("Player %1 (Id=%2)").arg(i + 1).arg(id));
+   d->mSideIndex2Side.insert(sides.count() - 1, id);
+ }
+
+ int watchId = 1; // ID of a player who watches the game only (but won't be able to play)
+ {
+   bool isTaken = false;
+   watchId = 1;
+   do {
+     isTaken = false;
+     QPtrListIterator<Player> it(*boGame->allPlayerList());
+     for (; it.current(); ++it) {
+       Player* p = it.current();
+       if (p->bosonId() == watchId && p != mSelectedPlayer) {
+         isTaken = true;
+       }
+     }
+     if (isTaken) {
+       watchId++;
+     }
+   } while (isTaken && watchId <= 127);
+ }
+
+ // AB: by definition a "watch" player must have an Id >= 1 and <= 127.
+ //     See http://boson.freehackers.org/wiki/Main/PlayerIds
+ if (watchId >= 1 && watchId <= 127) {
+   sides.append(i18n("Watch game only (Id = %1)").arg(watchId));
+   d->mSideIndex2Side.insert(sides.count() - 1, watchId);
+ }
+
+ mPlayerDesiredSide->setItems(sides);
+
+ if (d->mSideIndex2Side.count() != mPlayerDesiredSide->count()) {
+   boError() << k_funcinfo << "invalid side count" << endl;
+   d->mSideIndex2Side.clear();
+   mPlayerDesiredSide->clear();
+   return;
+ }
+
+ BO_CHECK_NULL_RET(mSelectedPlayer);
+ int selectedPlayerSide = mSelectedPlayer->bosonId();
+ for (QMap<int, int>::iterator it = d->mSideIndex2Side.begin(); it != d->mSideIndex2Side.end(); it++) {
+   if (it.data() == selectedPlayerSide) {
+     mPlayerDesiredSide->blockSignals(true);
+     mPlayerDesiredSide->setCurrentItem(it.key());
+     mPlayerDesiredSide->blockSignals(false);
+     break;
+   }
+ }
+}
+
 void BoUfoNewGameWidget::slotSetAdmin(bool admin)
 {
  boDebug() << k_funcinfo << endl;
  mStartGame->setEnabled(admin);
  mAddAIPlayer->setEnabled(admin);
- if (admin) {
-    mSelectMap->setSelectionMode(BoUfoListBox::SingleSelection);
- } else {
-    mSelectMap->setSelectionMode(BoUfoListBox::NoSelection);
- }
+ d->mPlayFieldSelection->setAdmin(admin);
 }
 
 void BoUfoNewGameWidget::slotOfferingConnections()
@@ -964,6 +1235,7 @@ void BoUfoNewGameWidget::removePlayer(KPlayer* p)
 
         // Update player info
         playerCountChanged();
+        possibleSidesChanged();
         slotPlayerSelected(mConnectedPlayersList->selectedItem());
         return;
     }
